@@ -29,6 +29,7 @@
 #define __CLUTTER_EVENT_H__
 
 #include <glib-object.h>
+#include <clutter/clutter-input-device.h>
 #include <clutter/clutter-types.h>
 
 #define CLUTTER_TYPE_EVENT	(clutter_event_get_type ())
@@ -106,6 +107,7 @@ typedef enum {
 
 /**
  * ClutterEventFlags:
+ * @CLUTTER_EVENT_NONE: No flag set
  * @CLUTTER_EVENT_FLAG_SYNTHETIC: Synthetic event
  *
  * Flags for the #ClutterEvent
@@ -113,6 +115,7 @@ typedef enum {
  * Since: 0.6
  */
 typedef enum {
+  CLUTTER_EVENT_NONE           = 0,
   CLUTTER_EVENT_FLAG_SYNTHETIC = 1 << 0,
 } ClutterEventFlags;
 
@@ -189,8 +192,6 @@ typedef enum
   CLUTTER_STAGE_STATE_ACTIVATED        = (1<<3)
 } ClutterStageState;
 
-typedef union _ClutterEvent ClutterEvent;
-
 typedef struct _ClutterAnyEvent         ClutterAnyEvent;
 typedef struct _ClutterButtonEvent      ClutterButtonEvent;
 typedef struct _ClutterKeyEvent         ClutterKeyEvent;
@@ -199,15 +200,6 @@ typedef struct _ClutterScrollEvent      ClutterScrollEvent;
 typedef struct _ClutterStageStateEvent  ClutterStageStateEvent;
 typedef struct _ClutterCrossingEvent    ClutterCrossingEvent;
 
-/**
- * ClutterInputDevice:
- *
- * Generic representation of an input device. The
- * actual contents of this structure depend on the
- * backend used.
- */
-typedef struct _ClutterInputDevice      ClutterInputDevice;
-
 /**
  * ClutterAnyEvent:
  * @type: event type
@@ -221,8 +213,8 @@ typedef struct _ClutterInputDevice      ClutterInputDevice;
  */
 struct _ClutterAnyEvent
 {
-  ClutterEventType  type;
-  guint32           time;
+  ClutterEventType type;
+  guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
@@ -252,6 +244,7 @@ struct _ClutterKeyEvent
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
+
   ClutterModifierType modifier_state;
   guint keyval;
   guint16 hardware_keycode;
@@ -290,13 +283,14 @@ struct _ClutterButtonEvent
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
-  gint x;
-  gint y;
+
+  gfloat x;
+  gfloat y;
   ClutterModifierType modifier_state;
   guint32 button;
   guint click_count;
   gdouble *axes; /* Future use */
-  ClutterInputDevice *device; /* Future use */
+  ClutterInputDevice *device;
 };
 
 /**
@@ -322,9 +316,10 @@ struct _ClutterCrossingEvent
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
-  gint x;
-  gint y;
-  ClutterInputDevice *device; /* future use */
+
+  gfloat x;
+  gfloat y;
+  ClutterInputDevice *device;
   ClutterActor *related;
 };
 
@@ -352,11 +347,12 @@ struct _ClutterMotionEvent
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
-  gint x;
-  gint y;
+
+  gfloat x;
+  gfloat y;
   ClutterModifierType modifier_state;
   gdouble *axes; /* Future use */
-  ClutterInputDevice *device; /* Future use */
+  ClutterInputDevice *device;
 };
 
 /**
@@ -384,12 +380,13 @@ struct _ClutterScrollEvent
   ClutterEventFlags flags;
   ClutterStage *stage;
   ClutterActor *source;
-  gint x;
-  gint y;
+
+  gfloat x;
+  gfloat y;
   ClutterScrollDirection direction;
   ClutterModifierType modifier_state;
   gdouble *axes; /* future use */
-  ClutterInputDevice *device; /* future use */
+  ClutterInputDevice *device;
 };
 
 /**
@@ -412,7 +409,8 @@ struct _ClutterStageStateEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source; /* unused XXX: should probably be the stage itself */
+  ClutterActor *source; /* XXX: should probably be the stage itself */
+
   ClutterStageState changed_mask;
   ClutterStageState new_state;
 };
@@ -440,33 +438,44 @@ union _ClutterEvent
 
 GType clutter_event_get_type (void) G_GNUC_CONST;
 
-gboolean            clutter_events_pending      (void);
-ClutterEvent *      clutter_event_get           (void);
-ClutterEvent *      clutter_event_peek          (void);
-void                clutter_event_put           (ClutterEvent       *event);
-ClutterEvent *      clutter_event_new           (ClutterEventType    type);
-ClutterEvent *      clutter_event_copy          (ClutterEvent       *event);
-void                clutter_event_free          (ClutterEvent       *event);
-ClutterEventType    clutter_event_type          (ClutterEvent       *event);
-guint32             clutter_event_get_time      (ClutterEvent       *event);
-ClutterModifierType clutter_event_get_state     (ClutterEvent       *event);
-void                clutter_event_get_coords    (ClutterEvent       *event,
-                                                 gint               *x,
-                                                 gint               *y);
-gint                clutter_event_get_device_id (ClutterEvent       *event);
-ClutterActor*       clutter_event_get_source    (ClutterEvent       *event);
+gboolean               clutter_events_pending               (void);
+ClutterEvent *         clutter_event_get                    (void);
+ClutterEvent *         clutter_event_peek                   (void);
+void                   clutter_event_put                    (ClutterEvent     *event);
 
-guint               clutter_key_event_symbol    (ClutterKeyEvent    *keyev);
-guint16             clutter_key_event_code      (ClutterKeyEvent    *keyev);
-guint32             clutter_key_event_unicode   (ClutterKeyEvent    *keyev);
+ClutterEvent *         clutter_event_new                    (ClutterEventType  type);
+ClutterEvent *         clutter_event_copy                   (ClutterEvent     *event);
+void                   clutter_event_free                   (ClutterEvent     *event);
 
-guint32             clutter_button_event_button (ClutterButtonEvent *buttev);
+ClutterEventType       clutter_event_type                   (ClutterEvent     *event);
+ClutterEventFlags      clutter_event_get_flags              (ClutterEvent     *event);
+guint32                clutter_event_get_time               (ClutterEvent     *event);
+ClutterModifierType    clutter_event_get_state              (ClutterEvent     *event);
+gint                   clutter_event_get_device_id          (ClutterEvent     *event);
+ClutterInputDeviceType clutter_event_get_device_type        (ClutterEvent     *event);
+ClutterInputDevice *   clutter_event_get_device             (ClutterEvent     *event);
+ClutterActor *         clutter_event_get_source             (ClutterEvent     *event);
+ClutterStage *         clutter_event_get_stage              (ClutterEvent     *event);
 
-guint32             clutter_keysym_to_unicode   (guint               keyval);
+void                   clutter_event_get_coords             (ClutterEvent     *event,
+                                                             gfloat           *x,
+                                                             gfloat           *y);
 
-ClutterStage*       clutter_event_get_stage     (ClutterEvent       *event);
+guint                  clutter_event_get_key_symbol         (ClutterEvent     *event);
+guint16                clutter_event_get_key_code           (ClutterEvent     *event);
+guint32                clutter_event_get_key_unicode        (ClutterEvent     *event);
 
-guint32             clutter_get_current_event_time (void);
+guint32                clutter_event_get_button             (ClutterEvent     *event);
+guint                  clutter_event_get_click_count        (ClutterEvent     *event);
+
+ClutterActor *         clutter_event_get_related            (ClutterEvent     *event);
+
+ClutterScrollDirection clutter_event_get_scroll_direction   (ClutterEvent     *event);
+
+guint32                clutter_keysym_to_unicode            (guint             keyval);
+
+guint32                clutter_get_current_event_time       (void);
+G_CONST_RETURN ClutterEvent *clutter_get_current_event      (void);
 
 G_END_DECLS
 
