@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authors:
  *  Neil Roberts   <neil@linux.intel.com>
@@ -286,6 +286,12 @@ _cogl_texture_2d_new_from_bitmap (CoglHandle       bmp_handle,
 
   g_return_val_if_fail (bmp_handle != COGL_INVALID_HANDLE, COGL_INVALID_HANDLE);
 
+  internal_format = _cogl_texture_determine_internal_format (bmp->format,
+                                                             internal_format);
+
+  if (!_cogl_texture_2d_can_create (bmp->width, bmp->height, internal_format))
+    return COGL_INVALID_HANDLE;
+
   if (!_cogl_texture_prepare_for_upload (bmp,
                                          internal_format,
                                          &internal_format,
@@ -344,13 +350,22 @@ _cogl_texture_2d_transform_coords_to_gl (CoglTexture *tex,
      anything */
 }
 
-static gboolean
+static CoglTransformResult
 _cogl_texture_2d_transform_quad_coords_to_gl (CoglTexture *tex,
                                               float *coords)
 {
   /* The texture coordinates map directly so we don't need to do
-     anything */
-  return TRUE;
+     anything other than check for repeats */
+
+  gboolean need_repeat = FALSE;
+  int i;
+
+  for (i = 0; i < 4; i++)
+    if (coords[i] < 0.0f || coords[i] > 1.0f)
+      need_repeat = TRUE;
+
+  return (need_repeat ? COGL_TRANSFORM_HARDWARE_REPEAT
+          : COGL_TRANSFORM_NO_REPEAT);
 }
 
 static gboolean
