@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __COGL_GLES2_WRAPPER_H__
@@ -37,6 +37,7 @@ typedef struct _CoglGles2WrapperTextureUnit
 
 typedef struct _CoglGles2WrapperAttributes  CoglGles2WrapperAttributes;
 typedef struct _CoglGles2WrapperUniforms  CoglGles2WrapperUniforms;
+typedef struct _CoglGles2WrapperTexEnv    CoglGles2WrapperTexEnv;
 typedef struct _CoglGles2WrapperTextureUnitSettings
 					  CoglGles2WrapperTextureUnitSettings;
 typedef struct _CoglGles2WrapperSettings  CoglGles2WrapperSettings;
@@ -54,14 +55,10 @@ typedef struct _CoglGles2WrapperShader	  CoglGles2WrapperShader;
 /* Accessors for the texture unit bit mask */
 #define COGL_GLES2_TEXTURE_UNIT_IS_ENABLED(mask, unit)  \
   (((mask) & (1 << ((unit) * 2))) ? TRUE : FALSE)
-#define COGL_GLES2_TEXTURE_UNIT_IS_ALPHA_ONLY(mask, unit)       \
-  (((mask) & (1 << ((unit) * 2 + 1))) ? TRUE : FALSE)
 #define COGL_GLES2_SET_BIT(mask, bit, val)                              \
   ((val) ? ((mask) |= (1 << (bit))) : ((mask) &= ~(1 << (bit))))
 #define COGL_GLES2_TEXTURE_UNIT_SET_ENABLED(mask, unit, val)    \
   COGL_GLES2_SET_BIT ((mask), (unit) * 2, (val))
-#define COGL_GLES2_TEXTURE_UNIT_SET_ALPHA_ONLY(mask, unit, val) \
-  COGL_GLES2_SET_BIT ((mask), (unit) * 2 + 1, (val))
 
 #define COGL_GLES2_MAX_TEXTURE_UNITS (sizeof (guint32) * 8 / 2)
 
@@ -115,6 +112,19 @@ struct _CoglGles2WrapperUniforms
   GLint      texture_unit_uniform;
 };
 
+struct _CoglGles2WrapperTexEnv
+{
+  GLenum texture_combine_rgb_func;
+  GLenum texture_combine_rgb_src[3];
+  GLenum texture_combine_rgb_op[3];
+
+  GLenum texture_combine_alpha_func;
+  GLenum texture_combine_alpha_src[3];
+  GLenum texture_combine_alpha_op[3];
+
+  GLfloat texture_combine_constant[4];
+};
+
 /* NB: We get a copy of this for each fragment/vertex
  * program varient we generate so we try to keep it
  * fairly lean */
@@ -130,6 +140,8 @@ struct _CoglGles2WrapperSettings
 
   unsigned int alpha_test_enabled:1;
   unsigned int fog_enabled:1;
+
+  CoglGles2WrapperTexEnv tex_env[COGL_GLES2_MAX_TEXTURE_UNITS];
 };
 
 struct _CoglGles2WrapperTextureUnit
@@ -301,6 +313,8 @@ struct _CoglGles2WrapperShader
 #define GL_EMISSION            0x1600
 #define GL_SHININESS           0x1601
 
+#define GL_MAX_TEXTURE_UNITS   0x84e2
+
 #endif /* GL_MODELVIEW */
 
 void cogl_gles2_wrapper_init (CoglGles2Wrapper *wrapper);
@@ -361,9 +375,6 @@ void cogl_wrap_glDrawElements (GLenum mode, GLsizei count, GLenum type,
                                const GLvoid *indices);
 void cogl_wrap_glTexParameteri (GLenum target, GLenum pname, GLfloat param);
 
-void cogl_gles2_wrapper_bind_texture (GLenum target, GLuint texture,
-				      GLenum internal_format);
-
 void cogl_wrap_glMaterialfv (GLenum face, GLenum pname, const GLfloat *params);
 
 /* This function is only available on GLES 2 */
@@ -412,11 +423,6 @@ void _cogl_gles2_clear_cache_for_program (CoglHandle program);
 
 #else /* HAVE_COGL_GLES2 */
 
-/* The extra third parameter of the bind texture wrapper isn't needed
-   so we can just directly call glBindTexture */
-#define cogl_gles2_wrapper_bind_texture(target, texture, internal_format) \
-  glBindTexture ((target), (texture))
-
 /* COGL uses the automatic mipmap generation for GLES 1 so
    glGenerateMipmap doesn't need to do anything */
 #define cogl_wrap_glGenerateMipmap(x) ((void) 0)
